/* This program reads a coded bib file and creates two files with authors data. One file is 
   the number of authors on each paper. The format of each line is 

   key\tnumberAuthors\tyear\n. 

   The other is number of papaers written by each author per year. The format of each line is 

   AuthorCode\tn_papers_y1\tn_papers_y2\t...n_papers_yk\n.

   The call is "AD bibfile prefix paperfile authorfile" where prefix is the three letter prefix
   of the author code*/



#include "stdio.h"
#include "ctype.h"
#include "string.h"

#define MAXLINE 400   /* Number of characters on a line */
#define MAXTITLESIZE 500 /* Max number of characters on a title */
#define ACODESZ 8 /* Author code size */
#define MAXCOAUTHORS 30 /* Max number of coauthors per article */
#define MAXKEY 12 /* Max length of key */

#define MAXAUTHORS 10000 /* Max number of authors in database*/

#define FIRSTYEAR 1996
#define LASTYEAR 2011
#define TOTALCOL LASTYEAR-FIRSTYEAR+1 /* Column for the total */

#define NUL '\0'
#define QTE '"' 

typedef int boolean;
#define TRUE 1
#define FALSE 0

char inpline[MAXLINE];
int cursor;

int lineno;

char title[MAXTITLESIZE]; /* The title */
char authorline[MAXLINE];

char prefix[MAXKEY];

int art_auth[MAXCOAUTHORS]; /*Author list for a publication*/

/*Publ per authors per year + total in last column*/
int au_publ[MAXAUTHORS][LASTYEAR-FIRSTYEAR+2];

int maxauth; /* Author with largest number*/

FILE * INPF; /* the input bib file */
FILE * PAPEROUTF; /* the paper data outfile */
FILE * AUTHOUTF;  /* The author count output file */


/* ------------ Functions ------------------- */

int readln(f,s)
FILE * f;
char *s;

/* Reads a line from file f and puts it into string s. Ends it with \0    */
/* If all is OK returns the number of characters in s, if EOF is reached, */
/* it returns 0, otherwise returns -1. Truncates line to MAXLINE          */
/* characters. MAXLINE is set to 80 if it is not defined previously       */

#ifndef MAXLINE
#define MAXLINE 80
#endif

{ int i, ok;
  char *p, ch;
  
  lineno++;
  
  p = s; i = 0;
  while (i < MAXLINE)
   { ok = fscanf(f,"%c",&ch);
     if (ok == 1) /* All is well */
       { if (ch == '\n') /* end of line */
           { *p = NUL;
             return(i+1);
            }
         *p++ = ch;
         i++;
       }
      else if (ok == EOF) 
            { *p = NUL;
              return(0);
             }
            else return(-1); /* ok has a strange value */ 
   }
  *p = NUL;
  /*****/printf("%d  ",lineno);
  /*****/if ((lineno % 10) == 0) printf("%d  ",lineno);
  return(MAXLINE);
}

/* ------------------ */
boolean isauthorline()

/* Returns TRUE if first non blank characters are 'AUTHOR', FALSE otherwise */

{ int i;
	
  i = 0;
  while(inpline[i] == ' ') i++; /* jump blanks */
  
  /* Just in case, check that we don't run out of characters...*/
  
  if ((inpline[i] == NUL) ||  (inpline[i+1] == NUL) || (inpline[i+2] == NUL) ||
	    (inpline[i+3] == NUL) || (inpline[i+4] == NUL) || (inpline[i+5] == NUL))
    return(FALSE);

  /* Is it? */
  
  if ((tolower(inpline[i]) == 'a') &&  (tolower(inpline[i+1]) == 'u') && 
       (tolower(inpline[i+2]) == 't') && (tolower(inpline[i+3]) == 'h') && 
       (tolower(inpline[i+4]) == 'o') && (tolower(inpline[i+5]) == 'r'))
    return(TRUE);

  /* No, it isn't */
  
  return(FALSE);
}
	

/* ------------------------------------ */

boolean istitleline()

/* Returns TRUE if first non blank characters are 'TITLE', FALSE otherwise */

{ int i;
	
  i = 0;
  while(inpline[i] == ' ') i++; /* jump blanks */
  
  /* Just in case, check that we don't run out of characters...*/
  
  if ((inpline[i] == NUL) ||  (inpline[i+1] == NUL) || (inpline[i+2] == NUL) ||
	    (inpline[i+3] == NUL) || (inpline[i+4] == NUL))
    return(FALSE);

  /* Is it? */
  
  if ((tolower(inpline[i]) == 't') &&  (tolower(inpline[i+1]) == 'i') && 
        (tolower(inpline[i+2]) == 't') && (tolower(inpline[i+3]) == 'l') &&
	    (tolower(inpline[i+4]) == 'e'))
    return(TRUE);

  /* No, it isn't */
  
  return(FALSE);
}
	
/* ------------------------------------ */

boolean isyearline()

/* Returns TRUE if first non blank characters are 'YEAR', FALSE otherwise */

{ int i;
	
  i = 0;
  while(inpline[i] == ' ') i++; /* jump blanks */
  
  /* Just in case, check that we don't run out of characters...*/
  
  if ((inpline[i] == NUL) ||  (inpline[i+1] == NUL) || (inpline[i+2] == NUL) ||
	    (inpline[i+3] == NUL))
    return(FALSE);

  /* Is it? */
  
  if ((tolower(inpline[i]) == 'y') &&  (tolower(inpline[i+1]) == 'e') && 
       (tolower(inpline[i+2]) == 'a') && (tolower(inpline[i+3]) == 'r'))
    return(TRUE);

  /* No, it isn't */
  
  return(FALSE);
}

/* ------------------------------------ */

void create_author_line()
/* Creates one author line from many author lines in bib file, it eliminates
 * AUTHOR = and " characters */

{ int i, j, aux;
  
  /* Read first line. It is somewhat special */
  
  i = 0; j = 0;
  while (inpline[i++] != QTE) ; /* jump AUTHOR = and so on */
  while (inpline[i] == ' ') i++; /* Jump blanks */

  /* Ready to copy */
  while ((inpline[i] != QTE) && (inpline[i] != NUL)) 
     authorline[j++] = inpline[i++];
  if (inpline[i] == QTE) /* Reached the end of entry */
    { authorline[j] = NUL;
	return;
    }
	
  /* It is only the end of the line. Read the rest */
  while (TRUE)
   { authorline[j++] = ' '; /* Add a blank just in case */
     aux = readln(INPF, inpline);
	 i = 0;
	 while (inpline[i] == ' ') i++; /* Jump blanks */
	 while ((inpline[i] != QTE) && (inpline[i] != NUL)) 
		  { authorline[j++] = inpline[i++];
		    if (j == MAXLINE-1)
				{ printf("Problem at line %d: too many authors\n",lineno);
			      authorline[j] = NUL;
				  return;
				}
		  }
	 if (inpline[i] == QTE) /* Reached the end of entry */
	   { authorline[j] = NUL;
	   return;
	   }
   } /* End of while */
}


/* ------------------------------------ */


void get_bibkey(s)
char s[];

/* Gets bib_ref key from input line, and puts it into s. */
/* Line always ends with ,                               */ 

{ int i,j;

  i = 0; j = 0;
  while (inpline[i++] != '{'); /* Go to key */
  while (inpline[i] == ' ') i++;
  
  while ((inpline[i] != ',') && (inpline[i] != ' ')) 
	   s[j++] = inpline[i++];
  s[j] = NUL;
}

/* ------------------------------------ */


int get_year()


/* Gets year  from input line, Already checked it is a YEAR line */

{ int i,j;

  i = 0; j = 0;
 
  /* go to first digit */
  while (!isdigit(inpline[i])) i++;
  j = inpline[i++] -'0';
  j = j*10 + (inpline[i++] -'0');
  j = j*10 + (inpline[i++] -'0');
  j = j*10 + (inpline[i] -'0');
  return(j);
}




/* ------------------------------------ */

int get_author_number()

/* Gets the number of the first author it finds from aline starting at cursor. 
Returns the number (from 0 to 9999) or -1 if there is no more authors*/

{ int n;

  /* Go to beginning of author code */
  while ((inpline[cursor] != prefix[0]) && (inpline[cursor] != NUL)) cursor++;
  
  if (inpline[cursor] == NUL) return(-1); /* No more authors*/
  /* Will not check rest of prefix. Beware*/
   n = (inpline[cursor+3] - '0')*1000 + (inpline[cursor+4] - '0')*100 +
         (inpline[cursor+5] - '0')*10 + (inpline[cursor+6] - '0');
		 
   cursor = cursor + 7; /*Prepared for next*/
   
   /* Update max author if necessary */
   if (n > maxauth) maxauth = n;
   
   return(n);
 }
   
   

/* ------------------------------------ */

int get_all_authors()

/* gets all autors in inpline[] and puts their numbers (from 1 to 9999) in
   art_auth[]. The rest of the entries are -1. Returns number of authors for the 
   publication. */
   
{ int i;
  int AuNr, n_of_auth; /* Author number and Number of authors in the publication */

  for (i = 0; i < MAXCOAUTHORS; i++) art_auth[i] = -1; /* Wipe out just in case */
  
  /* Go to first author*/
  
  cursor = 0;
  
  while (inpline[cursor++] != QTE);
  n_of_auth = 0;
  
  /* Get authors */
  AuNr = get_author_number();
  
  
  
  while (AuNr != -1)
    { art_auth[n_of_auth++] = AuNr;
	  AuNr = get_author_number();
	}

  return(n_of_auth);
}
  
	  
/* ------------------------------------ */



main(int argc, char *argv[])
{ 
  char  key[MAXKEY];
  int year;
  int col, totAuNr;
  
  boolean gkey, gaut, gtit, gyear;

  int auxl,i,j,k;

  
  maxauth = 0;
  
  gkey = FALSE;
  gaut = FALSE;
  gtit = FALSE;
  gyear = FALSE;
  
  if (argc != 5) 
    {printf("usage: %s prefix bibfile paperfile authorfile\n", argv[0] );
	exit(0);
  }
  
 
	strcpy(prefix,argv[1]);


  INPF = fopen(argv[2],"r");
	PAPEROUTF = fopen(argv[3],"w");   
   AUTHOUTF = fopen(argv[4],"w");   

  lineno = 0;
  
  /* Main loop */
  /* Go to first entry*/
  auxl = readln(INPF,inpline); 
  
  
  while (auxl != 0)
    { 
	  if (inpline[0] == '@') { get_bibkey(key);
								gkey = TRUE;/* printf("Got key \t");*/
							  }
							  
	  if (isyearline()) { year = get_year();
	                      gyear = TRUE;
						  /* printf("Got year \t");*/
						  }
	  if (istitleline())  /* We don't need the title*/
		  { gtit = TRUE; /* printf("passed title \t");*/
		  }
		  
		  
	  if (isauthorline())
	    { /*create_author_line();*/
          /* Put authors in table */
          
		  totAuNr = get_all_authors();
		  /* printf("Got authors: %d \t",totAuNr);*/
		  gaut = TRUE;
		}
		 
          
	  if (gkey && gyear && gtit  && gaut) /* Done reading the bib entry*/
		  { fprintf(PAPEROUTF,"%s\t%d\t%d\n", key, totAuNr, year);
		  
		     /**** printf("%s\t%d\t%d\n", key, totAuNr, year);*/
			
			col = year - FIRSTYEAR;
			for (j = 0; j<totAuNr; j++)
				{au_publ[art_auth[j]][col]++;
			     au_publ[art_auth[j]][TOTALCOL]++;
				 }
           /*** printf("Filled array\n");*/ 
			gkey = FALSE;
			gyear = FALSE;
			gtit = FALSE;
			gaut = FALSE;
		  }
	
	
	auxl = readln(INPF,inpline); 
	}
	
  /* Ready to print author information. First the header */
  for (k = FIRSTYEAR; k <= LASTYEAR; k++) fprintf(AUTHOUTF,"Y%d\t",k);
  fprintf(AUTHOUTF,"%s\n","Total");
  
  
  for (i = 1; i <= maxauth; i++)
	  { for (j = 0; j < TOTALCOL; j++)
		     fprintf(AUTHOUTF,"%d\t",au_publ[i][j]);
		fprintf(AUTHOUTF,"%d\n",au_publ[i][TOTALCOL]);
	   }
  
  fclose(INPF);
  fclose(PAPEROUTF);
  fclose(AUTHOUTF);

}  
