/* This program reads a bib file and converts it to two tables to be output
 * to two files.  The bib file has codes instead of author names "Autdddd".
 * The first table one is to be read by R. Its format is
 * "Aut1\tAut2\tArtcode\tyear\n" where Aut1 and Aut2 are author codes (with
 * Aut1 number < Aut2 number).  the artcode is the bib article code and year
 * is a number with the year of publication.  The second table is has the
 * format artcode\tFulltitle\n where fulltitle is the full title of the article*/



#include "stdio.h"
#include "ctype.h"
#include "string.h"

#define MAXLINE 400   /* Number of characters on a line */
#define MAXTITLESIZE 500 /* Max number of characters on a title */
#define ACODESZ 8 /* Author code size */
#define MAXCOAUTHORS 30 /* Max number of coauthors per article */
#define MAXKEY 12 /* Max length of key */

#define NUL '\0'
#define QTE '"' 

typedef int boolean;
#define TRUE 1
#define FALSE 0

/* 
 * struct article {
 *     char key[MAXKEY];
 *     int numa;   Number of authors of the article 
 * };
 */

char inpline[MAXLINE];
int cursor;

int lineno;

char title[MAXTITLESIZE]; /* The title */
char authorline[MAXLINE];

char prefix[4]; /* 3 letter author prefix*/

FILE * INPF; /* the input bib file */
FILE * AUTOUTF; /* the output edgeslist format file */
FILE * ARTTITOUTF;  /* The author count output file */


/* ------------ Functions ------------------- */

int readln(f,s)
FILE * f;
char *s;

/* Reads a line from file f and puts it into string s. Ends it with \0    */
/* If all is OK returns the number of characters in s, if EOF is reached, */
/* it returns 0, otherwise returns -1. Truncates line to MAXLINE          */
/* characters. MAXLINE is set to 80 if it is not defined previously       */

#ifndef MAXLINE
#define MAXLINE 80
#endif

{ int i, ok;
  char *p, ch;
  
  lineno++;
  
  p = s; i = 0;
  while (i < MAXLINE)
   { ok = fscanf(f,"%c",&ch);
     if (ok == 1) /* All is well */
       { if (ch == '\n') /* end of line */
           { *p = NUL;
             return(i+1);
            }
         *p++ = ch;
         i++;
       }
      else if (ok == EOF) 
            { *p = NUL;
              return(0);
             }
            else return(-1); /* ok has a strange value */ 
   }
  *p = NUL;
  return(MAXLINE);
}

/* ------------------ */
boolean isauthorline()

/* Returns TRUE if first non blank characters are 'AUTHOR', FALSE otherwise */

{ int i;
	
  i = 0;
  while(inpline[i] == ' ') i++; /* jump blanks */
  
  /* Just in case, check that we don't run out of characters...*/
  
  if ((inpline[i] == NUL) ||  (inpline[i+1] == NUL) || (inpline[i+2] == NUL) ||
	    (inpline[i+3] == NUL) || (inpline[i+4] == NUL) || (inpline[i+5] == NUL))
    return(FALSE);

  /* Is it? */
  
  if ((tolower(inpline[i]) == 'a') &&  (tolower(inpline[i+1]) == 'u') && 
       (tolower(inpline[i+2]) == 't') && (tolower(inpline[i+3]) == 'h') && 
       (tolower(inpline[i+4]) == 'o') && (tolower(inpline[i+5]) == 'r'))
    return(TRUE);

  /* No, it isn't */
  
  return(FALSE);
}
	

/* ------------------------------------ */

boolean istitleline()

/* Returns TRUE if first non blank characters are 'TITLE', FALSE otherwise */

{ int i;
	
  i = 0;
  while(inpline[i] == ' ') i++; /* jump blanks */
  
  /* Just in case, check that we don't run out of characters...*/
  
  if ((inpline[i] == NUL) ||  (inpline[i+1] == NUL) || (inpline[i+2] == NUL) ||
	    (inpline[i+3] == NUL) || (inpline[i+4] == NUL))
    return(FALSE);

  /* Is it? */
  
  if ((tolower(inpline[i]) == 't') &&  (tolower(inpline[i+1]) == 'i') && 
        (tolower(inpline[i+2]) == 't') && (tolower(inpline[i+3]) == 'l') &&
	    (tolower(inpline[i+4]) == 'e'))
    return(TRUE);

  /* No, it isn't */
  
  return(FALSE);
}
	
/* ------------------------------------ */

boolean isyearline()

/* Returns TRUE if first non blank characters are 'YEAR', FALSE otherwise */

{ int i;
	
  i = 0;
  while(inpline[i] == ' ') i++; /* jump blanks */
  
  /* Just in case, check that we don't run out of characters...*/
  
  if ((inpline[i] == NUL) ||  (inpline[i+1] == NUL) || (inpline[i+2] == NUL) ||
	    (inpline[i+3] == NUL))
    return(FALSE);

  /* Is it? */
  
  if ((tolower(inpline[i]) == 'y') &&  (tolower(inpline[i+1]) == 'e') && 
       (tolower(inpline[i+2]) == 'a') && (tolower(inpline[i+3]) == 'r'))
    return(TRUE);

  /* No, it isn't */
  
  return(FALSE);
}

/* ------------------------------------ */

void create_author_line()
/* Creates one author line from many author lines in bib file, it eliminates
 * AUTHOR = and " characters */

{ int i, j, aux;
  
  /* Read first line. It is somewhat special */
  
  i = 0; j = 0;
  while (inpline[i++] != QTE) ; /* jump AUTHOR = and so on */
  while (inpline[i] == ' ') i++; /* Jump blanks */

  /* Ready to copy */
  while ((inpline[i] != QTE) && (inpline[i] != NUL)) 
     authorline[j++] = inpline[i++];
  if (inpline[i] == QTE) /* Reached the end of entry */
    { authorline[j] = NUL;
	return;
    }
	
  /* It is only the end of the line. Read the rest */
  while (TRUE)
   { authorline[j++] = ' '; /* Add a blank just in case */
     aux = readln(INPF, inpline);
	 i = 0;
	 while (inpline[i] == ' ') i++; /* Jump blanks */
	 while ((inpline[i] != QTE) && (inpline[i] != NUL)) 
		  { authorline[j++] = inpline[i++];
		    if (j == MAXLINE-1)
				{ printf("Problem at line %d: too many authors\n",lineno);
			      authorline[j] = NUL;
				  return;
				}
		  }
	 if (inpline[i] == QTE) /* Reached the end of entry */
	   { authorline[j] = NUL;
	   return;
	   }
   } /* End of while */
}

/* ------------------------------------ */

void create_title_line()
/* Creates one title line from many title lines in bib file, it eliminates
 * TITLE = and " characters */

{ int i, j, aux;

  
  /* Read first line. It is somewhat special */
  
  i = 0; j = 0;
  while (inpline[i++] != QTE) ; /* jump TITLE = and so on */
  while ((inpline[i] == ' ') || (inpline[i] == '\t')) i++; /* Jump blanks */

  /* Ready to copy */
  while ((inpline[i] != QTE) && (inpline[i] != NUL)) 
     title[j++] = inpline[i++];
  if (inpline[i] == QTE) /* Reached the end of entry */
    { title[j] = NUL;
	  return;
    }
	
  /* It is only the end of the line. Read the rest */
  while (TRUE)
   { title[j++] = ' '; /* Add a blank just in case */
     aux = readln(INPF, inpline);
	 i = 0;
	 while ((inpline[i] == ' ') || (inpline[i] == '\t')) i++; /* Jump blanks */
	 while ((inpline[i] != QTE) && (inpline[i] != NUL)) 
		  { title[j++] = inpline[i++];
		    if (j == MAXTITLESIZE -1)
				{ printf("Problem at line %d: title too long\n",lineno);
			      title[j] = NUL;
				  return;
				}
		  }
	 if (inpline[i] == QTE) /* Reached the end of entry */
	   { title[j] = NUL;
	   return;
	   }
   } /* End of while */
}


/* ------------------------------------ */

int cmpaut(s1,s2)
char s1[];
char s2[];

/* returns 1 if s1 > s2, 2 otherwise: first three always prefix  */

{ 
  if (s1[3] > s2[3]) return(1);
  if (s2[3] > s1[3]) return(2);
  /* first digit equal */
  if (s1[4] > s2[4]) return(1);
  if (s2[4] > s1[4]) return(2);
  /* second digit also equal */
  if (s1[5] > s2[5]) return(1);
  if (s2[5] > s1[5]) return(2);
  /* third digit also equal */
  if (s1[6] > s2[6]) return(1);
  return(2);
}
 
/* ------------------------------------ */


void elim_and()
/* Eliminates the 'and' separator from the author line. Changes them to
 * '  #  ' */

{ int i;

  i = 0; 
  while (authorline[i+4] != NUL)
	{ if ((authorline[i] == ' ') && (authorline[i+1] == 'a') && 
            (authorline[i+2] == 'n') && (authorline[i+3] == 'd') && 
		       (authorline[i+4] == ' '))
	      { authorline[i+1] = ' '; authorline[i+2] = '#'; authorline[i+3] = ' '; 
		    i = i+5;	/* I assume next name is at least 5 char long */
			
	      } else i++;
	}
}



/* ------------------------------------ */

int get_next_author(a1)
char a1[];

/* gets name code from authorline and puts it into a1. The 
 * The code is 7 chars long. The function returns 
 * a 0 if it is the last name, a 1 otherwise */

{ int i;

/**printf("AL = %s cursor = %d\n",authorline, cursor);*/
  /* Put code in a1 Code is eight chars long (NUL Included)*/
  while (authorline[cursor] != prefix[0]) cursor++;
  for(i=0; i<7; i++) a1[i] = authorline[cursor++];
  a1[7] = NUL;
  
  /* Check if it is the last name of entry */
  while (authorline[cursor] == ' ') cursor++;
  if (authorline[cursor] == NUL) {/* printf("is last\n\n");*/
	return(0);
  }
  
  /* printf("is not last\n");*/
  return(1);
}


/* ------------------------------------ */


void get_bibkey(s)
char s[];

/* Gets bib_ref key from input line, and puts it into s. */
/* Line always ends with ,                               */ 

{ int i,j;

  i = 0; j = 0;
  while (inpline[i++] != '{'); /* Go to key */
  while (inpline[i] == ' ') i++;
  
  while ((inpline[i] != ',') && (inpline[i] != ' ')) 
	   s[j++] = inpline[i++];
  s[j] = NUL;
}

/* ------------------------------------ */


int get_year()


/* Gets year  from input line, Already checked it is a YEAR line */

{ int i,j;

  i = 0; j = 0;
 
  /* go to first digit */
  while (!isdigit(inpline[i])) i++;
  j = inpline[i++] -'0';
  j = j*10 + (inpline[i++] -'0');
  j = j*10 + (inpline[i++] -'0');
  j = j*10 + (inpline[i] -'0');
  return(j);
}

/* ------------------------------------ */



main(int argc, char *argv[])
{ 
  char autA[MAXCOAUTHORS][ACODESZ];     
  char  key[MAXKEY];
  int year;
  
  boolean gkey, gaut, gtit, gyear;

  int auxl,i,j,k;

  gkey = FALSE;
  gaut = FALSE;
  gtit = FALSE;
  gyear = FALSE;
  
  if (argc != 5) 
    {printf("usage: %s prefix BibFilename TableFileName ArtTitleFilename\n", argv[0] );
	exit(0);
  }
  
  
  strcpy(prefix, argv[1]);
  INPF = fopen(argv[2],"r");
  AUTOUTF = fopen(argv[3],"w");   
  ARTTITOUTF = fopen(argv[4],"w");   


  lineno = 0;
  
  /* Main loop */
  
  /* Go to first entry*/
  auxl = readln(INPF,inpline); 
  
  while (auxl != 0)
    { 
	  if (inpline[0] == '@') { get_bibkey(key);
	                           gkey = TRUE;
							  }
	  if (isyearline()) { year = get_year();
	                      gyear = TRUE;
						  }
	  if (istitleline())
	      { create_title_line();
		    gtit = TRUE;
		  }
		  
	  if (isauthorline())
	    { create_author_line();
          /* Put authors in table */
          i = 0;
		  cursor = 0;
          while (get_next_author(autA[i++]));
		  gaut = TRUE;
		}
		 
          
	  if (gkey && gyear && gtit  && gaut)
		  { fprintf(ARTTITOUTF,"%s\t%s\n", key, title);
	  
	        for (j = 0; j < i-1; j++)
              for (k = j+1; k < i; k++)
                { if (cmpaut(autA[j],autA[k]) == 1) /*autA[j] is smaller */
                   fprintf(AUTOUTF,"%s\t%s\t%s\t%d\n", autA[j], autA[k], key, year);
                  else 
                   fprintf(AUTOUTF,"%s\t%s\t%s\t%d\n", autA[k], autA[j], key, year);
            }
	        gkey = FALSE;
			gyear = FALSE;
			gtit = FALSE;
			gaut = FALSE;
		  }
	
	auxl = readln(INPF,inpline); 
	}
  
  fclose(INPF);
  fclose(ARTTITOUTF);
  fclose(AUTOUTF);

}  
